//
// Hilfsprogramme fr den DCF-77 Dekoder
//

// Arbeitsregister (werden ohne "r" geschrieben)

.extern DCF_fehler

// ***** 64 bit Schieberegister um eine Position weiterschieben *****
//	In dem Schieberegister werden die DCF77 Datenbits gesammelt
//	Parameter 1 wird in r24:r25 bergeben und enthlt die Schieberegisteradresse
//	Parameter 2 wird in r22:R23 bergeben und enthlt das Bit was eingeschoben werden soll
//  Die Parameter sind Word-"aligned" d.h. LSB immer in der geraden Registernummer.
//	Bei 8-Bit und 16-Bit Parametern somit beginnend bei r24 dann r22 dann r20 etc.
//	Rueckgabewert in r24:r25
//
.global schiebe
.func schiebe

schiebe:

	movw 30, 24			// Schieberegisteradresse nach Z laden
	tst 22				// einzuschiebendes Bit auf zero testen
	ldi 22, 8			// Wiederholungsanzahl laden
	mov 1, 22			// und in R1 ablegen
	clc					// carry lschen
	breq marke			// wenn einzuschiebendes Bit = zero, dann "0" einschieben
	sec					// sonst carry setzen und damit "1" einschieben
 marke:
		ld 22, Z			// Arbeitsregister mit Schieberegisterstelle laden
		ror 22				// carry einschieben und bertrag nach carry
		st Z+, 22			// Wert zurck in den Speicher und Zeiger auf die nchste Schieberegisterstelle
		dec 1				// Wiederholungszhler um 1 verringern
	brne marke				// zurck zum Schleifenanfang bis 0 erreicht ist

ret

.endfunc



// ***** Umwandlung von BCD in Binrdarstellung *****
//	BCD-Wert wird als Byte in r24 bernommen
//		Verfahren: Binrwert = BCD-Wert - 6 * Zehnerstelle des BCD-Wertes
//	Rckgabewert binre Zahl als Byte in r24
//
.global bcdtobin
.func bcdtobin

bcdtobin:

	mov		25, 24	// BCD-Wert kopieren
	swap	25		// durch 16 teilen
	andi	25, 15	// und Zehnerstelle mit 0F maskieren
	lsl		25		// Zehnerstelle * 2
	sub		24, 25	// zwischenwert = BCD - 2 * Zehnerstelle
	lsl		25		// Zehnerstelle * 4
	sub		24, 25	// binr = zwischenwert - 4 * Zehnerstelle
	
ret

.endfunc




// ***** Prfung der Paritt *****
//	Parameter 1 wird in r24:r25 bergeben und enthlt die Schieberegisteradresse
//	Parameter 2 wird in r22:R23 bergeben und enthlt die Anzahl des Bytes, die zu prfen sind
//	Rueckgabewert in r24:r25 0=gerade Paritt, 1=ungerade Paritt
//
.global paritaet
.func paritaet


paritaet:

	movW 30, 24			// Schieberegisteradresse nach Z laden
	ldi 24,0			// Ergebnis zurcksetzen
	
	byteloop:	
		ldi 25, 8		// Bitzhler = 8 
		ld 23, Z+		// aktuelles Byte nach R23
		
		bitloop:
			lsr	23			// Bit nach CY schieben 
			adc	24,25		// "1"-Bits aus CY addieren (r25 ist im Mittel gerade)
			dec 25			// Bitzhler um 1 verringern
		brne bitloop		// zurck zum Schleifenanfang bis 0 erreicht ist
		
		dec 22			// Byteanzahl verringern
	brne byteloop		// zurck zum Schleifenanfang bis 0 erreicht ist
	
	andi 24,1			// niederwertigste Stelle (gerade/ungerade) maskieren
	
ret

.endfunc



// ***** Lesen von 16 Byte fr den RAM-Speicherdump *****
//	Parameter 1 wird in r24:r25 bergeben und enthlt die Adresse des ersten zu lesenden Bytes
//	Parameter 2 wird in r22:R23 bergeben und zeigt auf das Datenfeld, in dem die Daten zurckgegeben werden.
//	kein Rueckgabewert
//
.global readRAM
.func readRAM


readRAM:

	push	r28
	push	r29
	
    movw	r30, r22	// Zeiger nach Z
    movw	r28, r24	// Adresse 1.Byte nach Y
    ldi		r22, 16		// Zhler auf 16 setzen
	
	rRloop:		
		ld		r24, Y+		// Byte lesen
		st		Z+, r24		// Byte an der Zeigerposition abspeichern 
		dec		r22			// Zhler-1
	brne	rRloop		// Wiederhole, bis Zhler = 0
	
	pop		r29
    pop		r28
	
ret
	
.endfunc

.end
