/**********************************************************
* driver for MCA-25 camera
* @Author: Simon Schulz [avr<AT>auctionant.de]
*
* ----------------------------------------------------------
* (Ingo Busker http://www.mikrocontroller.com):
* - new portpin mapping for LEDs, if ISA_CTRL is defined (in main.h)
* - you can disable LEDs
*-----------------------------------------------------------
* Joachim Brke
* ISA_CTRL ist inkonsistent zu ISP_CTRL
* deshalb zurckgendert in ISP_CTRL
***********************************************************/
#ifndef _MCA25_H_
	#define _MCA25_H_
	#include "main.h"


	#define USE_MCA25_CAM 0		//enable cam plugin
	
//	#define USE_WEBCAM_LEDS 1	
// Achtung: die LEDs werden im Hauptprogramm nicht untersttzt! 

	#define MCA25_COMM_BUFFER_LEN	40
	#define MCA25_NOT_BUSY			255
	#define MCA25_STILL_BUSY		254
	#define MCA25_FIRST_DATA		0
	#define MCA25_NEXT_DATA			1
	#define MCA25_SKIP_PICTURE		99
	
	//CONFIG:
#ifdef ISP_CTRL	
	#define MCA25_RESET_PORT     	PORTD
	#define MCA25_RESET_PORT_DIR 	DDRD
	
	#define MCA25_RESET_PIN      	6
#else
	#define MCA25_RESET_PORT     	PORTB
	#define MCA25_RESET_PORT_DIR 	DDRB
	#define MCA25_RESET_PIN      	0
#endif	
	//reset control signal
	#define MCA25_RESET_HI() MCA25_RESET_PORT|=(1<<MCA25_RESET_PIN);
	#define MCA25_RESET_LO() MCA25_RESET_PORT&=~(1<<MCA25_RESET_PIN);


#if USE_WEBCAM_LEDS

	#ifdef ISP_CTRL
		#define MCA25_STATUS_PIN     3	//status led 
		#define MCA25_ERROR_PIN      5	//error led 
		#define MCA25_CLOCK_PIN      4	//clock led 
	#else
		#define MCA25_STATUS_PIN     1	//status led 
		#define MCA25_ERROR_PIN      2	//error led 
		#define MCA25_CLOCK_PIN      4	//clock led 
	#endif	

	#define MCA25_CLOCK_LED_OFF()   MCA25_RESET_PORT|=(1<<MCA25_CLOCK_PIN);
	#define MCA25_CLOCK_LED_ON()    MCA25_RESET_PORT&=~(1<<MCA25_CLOCK_PIN);

	#define MCA25_ERROR_LED_OFF()   MCA25_RESET_PORT|=(1<<MCA25_ERROR_PIN);
	#define MCA25_ERROR_LED_ON()    MCA25_RESET_PORT&=~(1<<MCA25_ERROR_PIN);

	#define MCA25_STATUS_LED_OFF()  MCA25_RESET_PORT|=(1<<MCA25_STATUS_PIN);
	#define MCA25_STATUS_LED_ON()   MCA25_RESET_PORT&=~(1<<MCA25_STATUS_PIN);

#else
//	#define MCA25_STATUS_LED_OFF() nop();
//	#define MCA25_STATUS_LED_ON()  nop();

// Die Status_LED wird durch Modulation der roten LED an PD7 dargestellt
	#define MCA25_STATUS_LED_OFF() OCR2=VA2;
	#define MCA25_STATUS_LED_ON()  OCR2=0x7f;

	#define MCA25_ERROR_LED_OFF()  nop();
	#define MCA25_ERROR_LED_ON()   nop();
	
	#define MCA25_CLOCK_LED_OFF()  nop();
	#define MCA25_CLOCK_LED_ON()   nop();
#endif

	extern PROGMEM char MCA25_START_JPGa[];
	extern PROGMEM char MCA25_START_JPGb[];
	extern PROGMEM char MCA25_CONFIG_640x480a[];
	extern PROGMEM char MCA25_CONFIG_640x480b[];
	extern PROGMEM char MCA25_START_CAPTURING_1[];
	extern PROGMEM char MCA25_START_CAPTURING_2[];
	extern PROGMEM char MCA25_START_CAPTURING_3[];
	extern PROGMEM char MCA25_SEND_DATA_ACK[];
	extern PROGMEM char MCA25_REQUEST_CAMERA_INFO[];
	extern PROGMEM char MCA25_CONFIGURE_CAM_MODE[];

	extern volatile unsigned char mca25_uart_disabled;
	extern unsigned char mca25_cam_busy_for_socket;
	extern unsigned char mca25_cam_status;

	// functions:
	extern unsigned char mca25_copy_image_data_to_tcp_buffer(char *buffer, int *bufferlen);
	extern void mca25_grab_data(char *buffer, unsigned int *datalen, char *frametype);
	extern void mca25_grab_jpeg(void);
	extern void mca25_start_image_grab(void);
	extern void mca25_init(void);
	extern void mca25_configure(void);
	
	extern void mca25_send_data_ack(void);
	extern void mca25_read_at_command(unsigned char *buffer);
	extern void mca25_read_mux_packet(unsigned char *buffer);
	extern void mca25_send_ok(void);
	extern void mca25_pgm_send(PGM_P);
	extern void mca25_set_460800baud(void);
	
#endif
